@@ -1,11 +1,12 @@
 """Main module."""
 import asyncio
-from datetime import datetime, timedelta, timezone
+import json
 import logging
 import pathlib
+import re
 import shutil
-import json
 from asyncio.exceptions import TimeoutError
+from datetime import datetime, timedelta, timezone
 from typing import Callable, List, Optional
 
 import aiocron
@@ -175,6 +176,8 @@ class UnifiProtectBackup:
         rclone_args (str): Extra args passed directly to `rclone rcat`.
         ignore_cameras (List[str]): List of camera IDs for which to not backup events
         verbose (int): How verbose to setup logging, see :func:`setup_logging` for details.
+        detection_types (List[str]): List of which detection types to backup.
+        file_structure_format (str): A Python format string for output file path
         _download_queue (asyncio.Queue): Queue of events that need to be backed up
         _unsub (Callable): Unsubscribe from the websocket callback
         _has_ffprobe (bool): If ffprobe was found on the host
@@ -189,7 +192,9 @@ class UnifiProtectBackup:
         rclone_destination: str,
         retention: str,
         rclone_args: str,
+        detection_types: List[str],
         ignore_cameras: List[str],
+        file_structure_format: str,
         verbose: int,
         port: int = 443,
     ):
@@ -209,7 +214,9 @@ class UnifiProtectBackup:
                             (https://rclone.org/filtering/#max-age-don-t-transfer-any-file-older-than-this)
             rclone_args (str): A bandwidth limit which is passed to the `--bwlimit` argument of
                                    `rclone` (https://rclone.org/docs/#bwlimit-bandwidth-spec)
-            ignore_cameras (List[str]): List of camera IDs for which to not backup events
+            detection_types (List[str]): List of which detection types to backup.
+            ignore_cameras (List[str]): List of camera IDs for which to not backup events.
+            file_structure_format (str): A Python format string for output file path.
             verbose (int): How verbose to setup logging, see :func:`setup_logging` for details.
         """
         setup_logging(verbose)
@@ -229,10 +236,13 @@ class UnifiProtectBackup:
         logger.debug(f"  {rclone_args=}")
         logger.debug(f"  {ignore_cameras=}")
         logger.debug(f"  {verbose=}")
+        logger.debug(f"  {detection_types=}")
+        logger.debug(f"  {file_structure_format=}")
 
         self.rclone_destination = rclone_destination
         self.retention = retention
         self.rclone_args = rclone_args
+        self.file_structure_format = file_structure_format
 
         self.address = address
         self.port = port
@@ -251,6 +261,7 @@ class UnifiProtectBackup:
         self.ignore_cameras = ignore_cameras
         self._download_queue: asyncio.Queue = asyncio.Queue()
         self._unsub: Callable[[], None]
+        self.detection_types = detection_types
 
         self._has_ffprobe = False
 
@@ -420,8 +431,22 @@ class UnifiProtectBackup:
             return
         if msg.new_obj.end is None:
             return
-        if msg.new_obj.type not in {EventType.MOTION, EventType.SMART_DETECT}:
+        if msg.new_obj.type not in [EventType.MOTION, EventType.SMART_DETECT, EventType.RING]:
             return
+        if msg.new_obj.type is EventType.MOTION and "motion" not in self.detection_types:
+            logger.extra_debug(f"Skipping unwanted motion detection event: {msg.new_obj.id}")  # type: ignore
+            return
+        if msg.new_obj.type is EventType.RING and "ring" not in self.detection_types:
+            logger.extra_debug(f"Skipping unwanted ring event: {msg.new_obj.id}")  # type: ignore
+            return
+        elif msg.new_obj.type is EventType.SMART_DETECT:
+            for event_smart_detection_type in msg.new_obj.smart_detect_types:
+                if event_smart_detection_type not in self.detection_types:
+                    logger.extra_debug(  # type: ignore
+                        f"Skipping unwanted {event_smart_detection_type} detection event: {msg.new_obj.id}"
+                    )
+                    return
+
         self._download_queue.put_nowait(msg.new_obj)
         logger.debug(f"Adding event {msg.new_obj.id} to queue (Current queue={self._download_queue.qsize()})")
 
@@ -439,13 +464,16 @@ class UnifiProtectBackup:
 
                 # Fix timezones since pyunifiprotect sets all timestamps to UTC. Instead localize them to
                 # the timezone of the unifi protect NVR.
-                event.start.replace(tzinfo=pytz.utc).astimezone(self._protect.bootstrap.nvr.timezone)
-                event.end.replace(tzinfo=pytz.utc).astimezone(self._protect.bootstrap.nvr.timezone)
+                event.start = event.start.replace(tzinfo=pytz.utc).astimezone(self._protect.bootstrap.nvr.timezone)
+                event.end = event.end.replace(tzinfo=pytz.utc).astimezone(self._protect.bootstrap.nvr.timezone)
 
                 logger.info(f"Backing up event: {event.id}")
                 logger.debug(f"Remaining Queue: {self._download_queue.qsize()}")
                 logger.debug(f"  Camera: {await self._get_camera_name(event.camera_id)}")
-                logger.debug(f"  Type: {event.type}")
+                if event.type == EventType.SMART_DETECT:
+                    logger.debug(f"  Type: {event.type} ({', '.join(event.smart_detect_types)})")
+                else:
+                    logger.debug(f"  Type: {event.type}")
                 logger.debug(f"  Start: {event.start.strftime('%Y-%m-%dT%H-%M-%S')} ({event.start.timestamp()})")
                 logger.debug(f"  End: {event.end.strftime('%Y-%m-%dT%H-%M-%S')} ({event.end.timestamp()})")
                 duration = (event.end - event.start).total_seconds()
@@ -485,8 +513,10 @@ class UnifiProtectBackup:
                 if self._has_ffprobe:
                     try:
                         downloaded_duration = await self._get_video_length(video)
-                        msg = f"  Downloaded video length: {downloaded_duration:.3f}s" \
-                              f"({downloaded_duration - duration:+.3f}s)"
+                        msg = (
+                            f"  Downloaded video length: {downloaded_duration:.3f}s"
+                            f"({downloaded_duration - duration:+.3f}s)"
+                        )
                         if downloaded_duration < duration:
                             logger.warning(msg)
                         else:
@@ -567,12 +597,15 @@ class UnifiProtectBackup:
     async def generate_file_path(self, event: Event) -> pathlib.Path:
         """Generates the rclone destination path for the provided event.
 
-        Generates paths in the following structure:
-        ::
-          rclone_destination
-          |- Camera Name
-             |- {Date}
-                 |- {start timestamp} {event type} ({detections}).mp4
+        Generates rclone destination path for the given even based upon the format string
+        in `self.file_structure_format`.
+
+        Provides the following fields to the format string:
+          event: The `Event` object as per
+                 https://github.com/briis/pyunifiprotect/blob/master/pyunifiprotect/data/nvr.py
+          duration_seconds: The duration of the event in seconds
+          detection_type: A nicely formatted list of the event detection type and the smart detection types (if any)
+          camera_name: The name of the camera that generated this event
 
         Args:
             event: The event for which to create an output path
@@ -581,21 +614,23 @@ class UnifiProtectBackup:
             pathlib.Path: The rclone path the event should be backed up to
 
         """
-        path = pathlib.Path(self.rclone_destination)
         assert isinstance(event.camera_id, str)
-        path /= await self._get_camera_name(event.camera_id)  # directory per camera
-        path /= event.start.strftime("%Y-%m-%d")  # Directory per day
+        assert isinstance(event.start, datetime)
+        assert isinstance(event.end, datetime)
 
-        file_name = f"{event.start.strftime('%Y-%m-%dT%H-%M-%S')} {event.type}"
+        format_context = {
+            "event": event,
+            "duration_seconds": (event.end - event.start).total_seconds(),
+            "detection_type": f"{event.type} ({' '.join(event.smart_detect_types)})"
+            if event.smart_detect_types
+            else f"{event.type}",
+            "camera_name": await self._get_camera_name(event.camera_id),
+        }
 
-        if event.smart_detect_types:
-            detections = " ".join(event.smart_detect_types)
-            file_name += f" ({detections})"
-        file_name += ".mp4"
+        file_path = self.file_structure_format.format(**format_context)
+        file_path = re.sub(r'[^\w\-_\.\(\)/ ]', '', file_path)  # Sanitize any invalid chars
 
-        path /= file_name
-
-        return path
+        return pathlib.Path(f"{self.rclone_destination}/{file_path}")
 
     async def _get_camera_name(self, id: str):
         try:
